// -----------------------------------------------------------
// WSCommMonitor.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.controlpanel;

import cz.vutbr.wsagent.db.hibernate.DAOFactory;
import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import cz.vutbr.wsagent.entity.Message;
import java.util.ArrayList;
import java.util.List;

/**
 * The action class for monitoring all incoming messages.
 * @author Martin Gábor
 */
public class WSCommMonitor extends ControlPanelAction {

    private List<Message> messages = null;

    /**
     * Execute method.
     */
    @Override
    public String execute() {

        initMessageList();


        try {
            // fill in the messages
            List messagesList;

            messagesList = DAOFactory.getDAOFactory().getMessageDAO().readAll();
            setMessages(messagesList);


            // updates all that are new to be old
            DAOFactory.getDAOFactory().getMessageDAO().updateNewAll();

        } catch (DatabaseErrorException ex) {
            setResponseMessage(getText("Db.Error.message"));
        }


        return SUCCESS;
    }

    /** 
     * The initialisation of list.
     */
    private void initMessageList() {
        if (messages == null) {
            messages = new ArrayList();
        }
    }

    /**
     * Get the value of messages
     *
     * @return the value of messages
     */
    public List getMessages() {
        return messages;
    }

    /**
     * Set the value of messages
     *
     * @param messages new value of messages
     */
    public void setMessages(List messages) {
        this.messages = messages;
    }
}
