// -----------------------------------------------------------
// WSDrawTraces.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.controlpanel;

import cz.vutbr.wsagent.db.hibernate.DAOFactory;
import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import cz.vutbr.wsagent.util.LOGParser;
import cz.vutbr.wsagent.util.RSSIParser;
import cz.vutbr.wsagent.util.RSSIRecord;
import cz.vutbr.wsagent.util.TopologyMath;
import cz.vutbr.wsagent.util.TraceMath;
import cz.vutbr.wsagent.util.InvalidInputStrengthsException;
import cz.vutbr.wsagent.util.RectangleObject;
import cz.vutbr.wsagent.util.TraceDrawer;
import org.apache.struts2.ServletActionContext;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.io.IOException;
import cz.vutbr.wsagent.util.TracingAgent;

/**
 * The action class for displaying the movements.
 * @author Marek Houšť
 */
public class WSDrawTraces extends ControlPanelAction {

    // this identifiers determine the movements of agent
    private long agentClass;
    private long agentId;

    // records from with will be selected agent
    private List logsRecords;
    private List tracingAgentList;
    // records of the selected agent
    private TracingAgent tracingAgent;
    private long numberMovement;
    
    // image extension
    private final static String IMAGE_EXTENSION = ".jpg";
    private RSSIRecord topologyRecord;
    private String imageSource;
    
    /**
     * Execute method.
     */
    @Override
    public String execute() {
        
        // get list of new messages
        List messageList = null;
        try {
            messageList = DAOFactory.getDAOFactory().getMessageDAO().logReadAllBS(BASEST_ADDR);
        } catch (DatabaseErrorException ex) {
            setResponseMessage(getText("Db.Error.message"));
            return SUCCESS;
        }

        // class LOGParser is for parsing new message with logs
        LOGParser logParser = new LOGParser();
        // fill logsRecords
        setLogsRecords(logParser.getCommonLogDates(messageList));

        if (logsRecords.isEmpty()) {
            return SUCCESS;
        }
        
        // fill tracingAgentList
        setTracingAgentList(logParser.getTracingAgentList(logsRecords));
        // set tracingAgent under filled tracingAgentList and agentClass and agentId witch used select
        setTracingAgent(determinationTracingAgent());

        // class RSSIParser is for parsing new message with signal to surrounding neighbors
        RSSIParser parser = new RSSIParser();
        // set list of dates to the session
        setTopologyRecord(parser.getCommonRssiDatesForTraces(messageList));

        // Draw traces
        if (topologyRecord.getId() != -1) {
            // method to create image with agent movements
            drawTopologyWithTraces();

        }

        return SUCCESS;
    }

    /**
     * Get the value of agentClass
     *
     * @return the value of agentClass
     */
    public long getAgentClass() {
        return agentClass;
    }

    /**
     * Set the value of agentClass
     *
     * @param agentClass new value of agentClass
     */
    public void setAgentClass(long agentClass) {
        this.agentClass = agentClass;
    }

    /**
     * Get the value of agentId
     *
     * @return the value of agentId
     */
    public long getAgentId() {
        return agentId;
    }

    /**
     * Set the value of agentId
     *
     * @param agentId new value of agentId
     */
    public void setAgentId(long agentId) {
        this.agentId = agentId;
    }

    /**
     * Get the value of numberMovement
     *
     * @return the value of numberMovement
     */
    public long getNumberMovement() {
        return numberMovement;
    }

    /**
     * Set the value of numberMovement
     *
     * @param numberMovement new value of numberMovement
     */
    public void setNumberMovement(long numberMovement) {
        this.numberMovement = numberMovement;
    }

    /**
     * Get the value of logsRecords
     *
     * @return the value of logsRecords
     */
    public List getLogsRecords() {
        return logsRecords;
    }

    /**
     * Set the value of logsRecords
     *
     * @param logsRecords new value of logsRecords
     */
    public void setLogsRecords(List logsRecords) {
        this.logsRecords = logsRecords;
    }

    /**
     * Get the value of tracingAgentList
     *
     * @return the value of tracingAgentList
     */
    public List getTracingAgentList() {
        return tracingAgentList;
    }

    /**
     * Set the value of tracingAgentList
     *
     * @param tracingAgentList new value of tracingAgentList
     */
    public void setTracingAgentList(List tracingAgentList) {
        this.tracingAgentList = tracingAgentList;
    }    

    /**
     * Set the value of topologyRecord
     *
     * @param topologyRecord new value of topologyRecord
     */
    public void setTopologyRecord(RSSIRecord topologyRecord) {
        this.topologyRecord = topologyRecord;
    }

    /**
     * Get the value of imageSource
     *
     * @return the value of imageSource
     */
    public String getImageSource() {
        return imageSource;
    }

    /**
     * Set the value of imageSource
     *
     * @param imageSource new value of imageSource
     */
    public void setImageSource(String imageSource) {
        this.imageSource = imageSource;
    }

    /**
     * Get the value of tracingAgent
     *
     * @return the value of tracingAgent
     */
    public TracingAgent getTracingAgent() {
        return tracingAgent;
    }

    /**
     * Set the value of tracingAgent
     *
     * @param tracingAgent new value of tracingAgent
     */
    public void setTracingAgent(TracingAgent tracingAgent) {
        this.tracingAgent = tracingAgent;
    }

    /**
     * Select the tracingAgent from the tracingAgentList using the agentId and the agentClass.
     * @return newTracingAgent
     */
    private TracingAgent determinationTracingAgent() {

        TracingAgent newTracingAgent = null;
        for(int i = 0; i < tracingAgentList.size(); i++) {
            newTracingAgent = (TracingAgent) tracingAgentList.get(i);
            if ((newTracingAgent.getAgentClass() == getAgentClass()) && (newTracingAgent.getAgentId() == getAgentId())) {
                break;
            }
        }

        setNumberMovement(newTracingAgent.getTraceList().size());

        return newTracingAgent;
    }

    /**
     * Method for create the image with the agent movements.
     */
    private void drawTopologyWithTraces() {
        
        // to save the image
        String sourcePath = DAOFactory.getDAOFactory().getSettingDAO().read().getTopologyImgPath();

        /* GETTING THE SOURCE PATH */
        // the absolute webapp path
        String input = ServletActionContext.getServletContext().getRealPath("");

        // the path of image;
        String imagePath = sourcePath + "traces" + IMAGE_EXTENSION;
        setImageSource(imagePath);

        // we will delete image, if the file exists.
        if(new File(input + imagePath).exists()) {
            new File(input + imagePath).delete();
        }

        /*
         * Compute topology
         * ------------------
         */
        TopologyMath topologyMath = new TopologyMath();
        List objectList = new ArrayList();
        try {
            // application TopologyMath for calculating the coordinates
            objectList = topologyMath.getTopologyPoints(topologyRecord.getRssiObjectList());

        } catch (InvalidInputStrengthsException ex) {
            setResponseMessage(getText("UnableLoadImage.messagedrawtraces"));
        }

        TraceMath traceMath = new TraceMath();
        List trackingAgentObjects = new ArrayList();
        try {
            // calculating the coordinates for drawing objects
            trackingAgentObjects = traceMath.getTracePoints(getTracingAgent(), objectList);

        } catch (InvalidInputStrengthsException ex) {
            setResponseMessage(getText("UnableLoadImage.messagedrawtraces"));
        }

        // get border
        RectangleObject border = topologyMath.getWindowBorder(objectList);
        int width = border.getX2() - border.getX1();
        int height = border.getY2() - border.getY1();
        try {
            // check whether the class TopologyMath calculated good coordinates
            traceMath.checkWidthHeight(width, height);

        } catch (InvalidInputStrengthsException ex) {
            setResponseMessage(getText("UnableLoadImage.messagedrawtraces"));
            return;
        }

        /*
         * Draw traces
         * -----------
         */
        TraceDrawer traceDrawer = new TraceDrawer(width, height, input);
        try {
            // preparation picture
            traceDrawer.drawTraces(trackingAgentObjects, border);
            // creating picture
            traceDrawer.generateTopologyImage(getImageSource());

        } catch (IOException ex) {
            setResponseMessage(getText("InvalidImagesSourcePath.message"));
        } catch (NullPointerException ex) {
            setResponseMessage(getText("InvalidImagesSourcePath.message"));
        }

    }

}
