// -----------------------------------------------------------
// WSMessageSend.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.controlpanel;

import cz.vutbr.wsagent.client.DeviceBusyException;
import cz.vutbr.wsagent.client.TimeExpiredException;
import cz.vutbr.wsagent.controlpanel.exception.SendingMessageException;
import cz.vutbr.wsagent.controlpanel.exception.InvalidParametersException;
import cz.vutbr.wsagent.client.ClientSocket;
import cz.vutbr.wsagent.client.CommunicationErrorException;
import cz.vutbr.wsagent.client.MessageConstructor;
import cz.vutbr.wsagent.db.hibernate.DAOFactory;
import java.io.IOException;
import java.net.ConnectException;
import java.net.UnknownHostException;

/**
 * The action class for sending the messages.
 * @author Martin Gábor
 */
public class WSMessageSend extends ControlPanelAction {

    private String message;
    private String address;
    private String paramSend;

    /**
     * Execute method.
     */
    @Override
    public String execute() {

        if (paramSend != null) {

            setResponseMessage(getText("Comm.Success.message"));
            try {
                send();
            } catch (InvalidParametersException ex) {
                setResponseMessage(getText("InvalidParameters.message"));
            } catch (ConnectException ex) {
                setResponseMessage(getText("Comm.ConnectError.message"));
            } catch (SendingMessageException ex) {
                setResponseMessage(getText("Comm.SendingError.message"));
            } catch (TimeExpiredException ex) {
                setResponseMessage(getText("Comm.TimeExpired.message"));
            } catch (DeviceBusyException ex) {
                setResponseMessage(getText("Comm.DeviceBusy.message"));
            }
        }
        return SUCCESS;
    }

    /**
     * Method for sending messages.
     * @throws ConnectException Thrown if can not connect to BSComm application.
     * @throws InvalidParametersException Thrown when parameters are not valid.
     * @throws SendingMessageException Thrown when sending is not successful.
     * @throws TimeExpiredException Thrown when time of sending message expired.
     * @throws DeviceBusyException Thrown when mote is busy. There is a communication with basestation.
     */
    private void send() throws ConnectException, SendingMessageException, InvalidParametersException,
            TimeExpiredException, DeviceBusyException {


        // checking Parameters
        if (message == null || address == null) {
            throw new InvalidParametersException();
        }
        // still checking
        try {
            Integer.valueOf(address);
        } catch (NumberFormatException e) {
            throw new InvalidParametersException();
        }


        // get port of Socket communication
        int port = DAOFactory.getDAOFactory().getSettingDAO().read().getPort();

        // contruct new message to send
        MessageConstructor messageConstructor = new MessageConstructor();
        String simpleMessage = messageConstructor.constructSimpleMessage(Integer.valueOf(address), message);

        // sending
        ClientSocket clientSocket = null;
        try {
            clientSocket = new ClientSocket(port);
        } catch (UnknownHostException ex) {
            throw new ConnectException();
        } catch (IOException ex) {
            throw new ConnectException();
        }
        
        if (clientSocket != null) {
            try {
                clientSocket.send(simpleMessage);
                clientSocket.close();
            } catch (IOException ex) {
                throw new SendingMessageException();
            } catch (ClassNotFoundException ex) {
                throw new SendingMessageException();
            } catch (CommunicationErrorException ex) {
                throw new SendingMessageException();
            }
        } else {
            throw new ConnectException();
        }
    }

    public String getAddress() {
        return address;
    }

    public void setAddress(String address) {
        this.address = address;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public String getParamSend() {
        return paramSend;
    }

    public void setParamSend(String paramSend) {
        this.paramSend = paramSend;
    }
}
