// -----------------------------------------------------------
// WSMessages.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.controlpanel;

import cz.vutbr.wsagent.db.hibernate.DAOFactory;
import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import cz.vutbr.wsagent.entity.Mote;
import java.util.ArrayList;
import java.util.List;

/**
 * The action class for presenting messagess of some concrete mote.
 * @author Martin Gábor
 */
public class WSMessages extends ControlPanelAction {

    private List messages = null;
    private String moteAddress;
    private Mote mote;


    /**
     * Execute method.
     */
    @Override
    public String execute() {

        initMessageList();


        try {
            if (mote == null) {
                mote = DAOFactory.getDAOFactory().getMoteDAO().read(moteAddress);
            }

            // fill in the messages
            List messagesList;

            messagesList = DAOFactory.getDAOFactory().getMessageDAO().readAllForMote(mote);
            setMessages(messagesList);


            // updates all that are new to be old
            DAOFactory.getDAOFactory().getMessageDAO().updateNew(mote);

        } catch (DatabaseErrorException ex) {
            setResponseMessage(getText("Db.Error.message"));
        }


        return SUCCESS;
    }


    /**
     * The initialisation method for list of messages.
     */
    private void initMessageList() {
        if (messages == null) {
            messages = new ArrayList();
        }
    }

    /**
     * Get the value of messages
     *
     * @return the value of messages
     */
    public List getMessages() {
        return messages;
    }

    /**
     * Set the value of messages
     *
     * @param messages new value of messages
     */
    public void setMessages(List messages) {
        this.messages = messages;
    }

    /**
     * Get the value of moteAddress
     *
     * @return the value of moteAddress
     */
    public String getMoteAddress() {
        return moteAddress;
    }

    /**
     * Set the value of moteAddress
     *
     * @param address new value of moteAddress
     */
    public void setMoteAddress(String address) {
        this.moteAddress = address;
    }

    /**
     * Get the value of mote
     *
     * @return the value of mote
     */
    public Mote getMote() {
        return mote;
    }

    /**
     * Set the value of mote
     *
     * @param mote new value of mote
     */
    public void setMote(Mote mote) {
        this.mote = mote;
    }
}
