// -----------------------------------------------------------
// WSRequestTopology.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.controlpanel;

import cz.vutbr.wsagent.client.ClientSocket;
import cz.vutbr.wsagent.client.MessageConstructor;
import cz.vutbr.wsagent.client.TimeExpiredException;
import cz.vutbr.wsagent.db.hibernate.DAOFactory;
import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import cz.vutbr.wsagent.entity.Agent;
import cz.vutbr.wsagent.entity.Mote;
import java.util.Iterator;
import java.util.List;

/**
 * The action class for requesting topology graph.
 * @author Martin Gábor
 */
public class WSRequestTopology extends ControlPanelAction {

    private String request;


    /**
     * Execute method.
     */
    @Override
    public String execute() {

        // gets port
        int port = DAOFactory.getDAOFactory().getSettingDAO().read().getPort();

        // creates new message according to protocol
        MessageConstructor messageConstructor = new MessageConstructor();

        // list of all motes
        List moteList = null;
        // getting all Motes from DB
        try{
            moteList = DAOFactory.getDAOFactory().getMoteDAO().readAll();
        }catch(DatabaseErrorException e){
           return SUCCESS;
        }

        // creating new connector
        ClientSocket clientSocket = null;


        // looking for motes that are online
        // --------------------------------------------------------------
        for (Iterator mote = moteList.iterator(); mote.hasNext();) {
            Mote existingMote = (Mote) mote.next();

            // new agent with given request
            Agent agent = new Agent();
            agent.setPlan(request);

            String messageToSend = messageConstructor.constructAgentMessage(
                    Integer.valueOf(existingMote.getAddress()), agent);
            try {
                clientSocket = new ClientSocket(port);
                clientSocket.send(messageToSend);
                clientSocket.close();

            }  catch (TimeExpiredException e){
                continue;
            } catch (Exception ex) {
                break;
            }
        }
        // ---------------------------------------------------------------
        return SUCCESS;
    }

    /**
     * Get the value of request
     *
     * @return the value of request
     */
    public String getRequest() {
        return request;
    }

    /**
     * Set the value of request
     *
     * @param request new value of request
     */
    public void setRequest(String request) {
        this.request = request;
    }


}
