// -----------------------------------------------------------
// WSRequestTracingAgents.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.controlpanel;

import cz.vutbr.wsagent.db.hibernate.DAOFactory;
import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import cz.vutbr.wsagent.client.MessageConstructor;
import cz.vutbr.wsagent.client.ClientSocket;
import cz.vutbr.wsagent.client.TimeExpiredException;
import cz.vutbr.wsagent.entity.Agent;
import cz.vutbr.wsagent.entity.Mote;

/**
 * The action class for creating requirement to collection of records from the sensor nodes.
 * @author Marek Houšť
 */
public class WSRequestTracingAgents extends ControlPanelAction {

    // address of initial node
    private int addressMoteToRequest;
    // collection is performed by agent (his identification)
    private int agentIdToRequest;
    private int agentClassToRequest;

    private int sendOk;
    
    /**
     * Execute method.
     */
    @Override
    public String execute() {

        sendOk = 1;

        // change of new message to old message (this new messages have been collected in the previous request)
        try {

            DAOFactory.getDAOFactory().getMessageDAO().updateNewAll();

        } catch (DatabaseErrorException ex) {
            setResponseMessage(getText("Db.Error.message"));
            return SUCCESS;
        }
        // save initial node
        saveRequestMote(Integer.toString(addressMoteToRequest));
        // send request
        sendRequirementLOG();

        return SUCCESS;
    }

    /**
     * Get the value of addressMoteToRequest
     *
     * @return the value of addressMoteToRequest
     */
    public int getAddressMoteToRequest() {
        return addressMoteToRequest;
    }

    /**
     * Set the value of addressMoteToRequest
     *
     * @param addressMoteToRequest new value of addressMoteToRequest
     */
    public void setAddressMoteToRequest(int addressMoteToRequest) {
        this.addressMoteToRequest = addressMoteToRequest;
    }

    /**
     * Get the value of agentIdToRequest
     *
     * @return the value of agentIdToRequest
     */
    public int getAgentIdToRequest() {
        return agentIdToRequest;
    }

    /**
     * Set the value of agentIdToRequest
     *
     * @param agentIdToRequest new value of agentIdToRequest
     */
    public void setAgentIdToRequest(int agentIdToRequest) {
        this.agentIdToRequest = agentIdToRequest;
    }

    /**
     * Get the value of agentClassToRequest
     *
     * @return the value of agentClassToRequest
     */
    public int getAgentClassToRequest() {
        return agentClassToRequest;
    }

    /**
     * Set the value of agentClassToRequest
     *
     * @param agentClassToRequest new value of agentClassToRequest
     */
    public void setAgentClassToRequest(int agentClassToRequest) {
        this.agentClassToRequest = agentClassToRequest;
    }

    /**
     * Get the value of sendOk
     *
     * @return the value of sendOk
     */
    public int getSendOk() {
        return sendOk;
    }

    /**
     * Set the value of sendOk
     *
     * @param sendOk new value of sendOk
     */
    public void setSendOk(int sendOk) {
        this.sendOk = sendOk;
    }

    /**
     * Method for storing initial node.
     * @param addressMoteToRequest
     */
    private void saveRequestMote(String addressMoteToRequest) {

        Mote mote = null;
        try {
            // for test if mote is already in the database
            mote = DAOFactory.getDAOFactory().getMoteDAO().read(addressMoteToRequest);
        } catch (DatabaseErrorException ex) {
            setResponseMessage(getText("Db.Error.message"));
        }
        // test if mote is already in the database
        if (mote == null) {
            mote = new Mote();
            mote.setAddress(addressMoteToRequest);
            mote.setDescription("added by tracing agents request");
            DAOFactory.getDAOFactory().getMoteDAO().save(mote);
        }
    }

    /**
     * Metod for sending request (Send agent who made ​​the request)
     */
    private void sendRequirementLOG() {

        String requestTracingAgentsString_Plan = "^(cl)";
        
        String requestTracingAgentsString_PlanBase = "(cl,(&(1)$(t,(i))+(&1,v)&(3)$(e,(z))+(sn)$(n,m)^(cg)^(nb)))"
                + "(cg,($(g,(&3,10))$(l,(g,1))^(ig)$(l,(g,0))&(2)$(o,add,&3,(10),(),(),())&(3)$(f,&2)^(cg)))"
                + "(ig,(+(ig)&(1)$(t,(i))&(2)*(mr,&1)^(sg)^(sn)-(sn)-(ig)#&(2)*(ig)-(ig)^(ch)))"
                + "(sg,(&(2)*(LOG,_,_,_,_,_)&(1)$(f,&2)-&1!(1,&1)^(sg)))"
                + "(sn,(&(2)*(sn)&(2)*(M,_,_,_)&(1)$(f,&2)-&1!(1,&1)^(sn)))"
                + "(ch,(+(ch)+(&1,n)&(1)$(t,(b,l))$(c,(i,i))$(m,(&1))&(1)$(t,(i))&(2)*(&1,n)-(&1,n)-(ch)^(wt)$(c,(d,i))-(LOG,_,_,_,_,_)-(M,_,_,_)#&(2)*(ch)-(ch)&(2)$(t,(b,l))-(&2,v)-(&1,v)^(im)))"
                + "(wt,(+(wp)$(s)&(1)?(_)&(2)*(&1)-(wp)#*(wp)-(wp)^(wt)))"
                + "(im,($(l,(y,1))+(im)&(2)*(mr,&1)^(sg)^(sn)-(sn)-(im)^(lb)#&(2)*(im)-(im)^(lm)^(im)))"
                + "(lm,(&(2)*(_,v)^(l)&(3)$(f,&2)+(&1,b)&(1)$(f,&3)-&3$(m,(&1,s))))"
                + "(lb,($(l,(y,0))&(2)*(_,b)^(l)&(3)$(f,&2)&(1)$(f,&3)-&3$(m,(&1,s))^(lb)#&(1)*(_,n)&(2)$(f,&1)&(1)$(f,&2)!(&1,wp)$(k)))"
                + "(l,(&(3)$(r,&2)$(e,(e,&3))&(2)$(e,(c,&3))^(l)))"
                + "(nb,($(l,(r,1))$(n)^(cn)))"
                + "(cn,(^(gn)^(gi)^(tm)#^(cn)))"
                + "(gn,(+(bk)&(1)*(NB,_,_)$(l,(r,0))&(2)$(f,&1)-&2-(bk)#&(1)*(bk)-(bk)^(bk)))"
                + "(gi,(&(1)$(r,&2)&(2)$(f,&1)))"
                + "(tm,($(t,(q,&2))-(NB,_,_)$(m,(&2,s))^(cl)))"
                + "(bk,(&(1)$(t,(b,f))+(b,&1)&(2)*(b,(1))!(1,endlog)$(k)#&(2)$(t,(i))-(&2,v)$(m,(&1,s))^(nb)))";

        String requestTracingAgentsString_BB = "(mr,(" + addressMoteToRequest + "))";

        // gets port
        int port = DAOFactory.getDAOFactory().getSettingDAO().read().getPort();

        // creates new message according to protocol
        MessageConstructor messageConstructor = new MessageConstructor();

        // creating new connector
        ClientSocket clientSocket = null;

        // new agent with given request
        Agent agent = new Agent();
        agent.setPlanBase(requestTracingAgentsString_PlanBase);
        agent.setPlan(requestTracingAgentsString_Plan);
        agent.setBeliefBase(requestTracingAgentsString_BB);
        agent.setAgentId(Long.valueOf(agentIdToRequest));
        agent.setClassId(Long.valueOf(agentClassToRequest));

        String messageToSend = messageConstructor.constructAgentMessage(addressMoteToRequest, agent);

        try {
            clientSocket = new ClientSocket(port);
            clientSocket.send(messageToSend);
            clientSocket.close();
        }  catch (TimeExpiredException e){
            sendOk = 2;
        } catch (Exception ex) {
            
        }
    }

}
