// -----------------------------------------------------------
// WSSaveMote.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.controlpanel;

import cz.vutbr.wsagent.db.hibernate.DAOFactory;
import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import cz.vutbr.wsagent.entity.Mote;

/**
 * The action class for saving motes.
 * @author Martin Gábor
 */
public class WSSaveMote extends ControlPanelAction {

    private Mote moteBean;
    private String moteAddress;

    /**
     * Execute method.
     */
    @Override
    public String execute() {


        // if moteAddress is null - mote doesnt exist in a database
        if (moteAddress == null) {
            DAOFactory.getDAOFactory().getMoteDAO().save(moteBean);

        }
        // else mote probably exists in a database.
        else{
            Mote mote = null;
            try {
                // get existing mote from database
                mote = DAOFactory.getDAOFactory().getMoteDAO().read(moteAddress);

            } catch (DatabaseErrorException ex) {
                 return INPUT;
            }

            if (mote == null){
                setResponseMessage(getText("Db.Error.message"));
                return SUCCESS;
            }

            // set new Values to existing object.
            mote.setName(moteBean.getName());
            mote.setDescription(moteBean.getDescription());

            // update mote
            DAOFactory.getDAOFactory().getMoteDAO().update(mote);
        }

        return SUCCESS;

    }

    @Override
    public void validate() {
        if (getMoteAddress() == null){
            Mote mote = null;
            try {
                // check if mote exists in the database
                mote = DAOFactory.getDAOFactory().getMoteDAO().read(moteBean.getAddress());
            } catch (DatabaseErrorException ex) {
            }

            if (mote != null)
                addFieldError("moteBean.address", "Mote with given address already exists!");
        }
    }


    /**
     * Get the value of moteBean
     *
     * @return the value of moteBean
     */
    public Mote getMoteBean() {
        return moteBean;
    }

    /**
     * Set the value of moteBean
     *
     * @param moteBean new value of moteBean
     */
    public void setMoteBean(Mote moteBean) {
        this.moteBean = moteBean;
    }

    /**
     * Get the value of moteAddress
     *
     * @return the value of moteAddress
     */
    public String getMoteAddress() {
        return moteAddress;
    }

    /**
     * Set the value of moteAddress
     *
     * @param moteAddress new value of moteAddress
     */
    public void setMoteAddress(String moteAddress) {
        this.moteAddress = moteAddress;
    }
}
