// -----------------------------------------------------------
// WSTopology.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.controlpanel;

import cz.vutbr.wsagent.db.hibernate.DAOFactory;
import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import cz.vutbr.wsagent.util.RectangleObject;
import cz.vutbr.wsagent.util.InvalidInputStrengthsException;
import cz.vutbr.wsagent.util.TopologyMath;
import cz.vutbr.wsagent.util.RSSIParser;
import cz.vutbr.wsagent.util.RSSIRecord;
import cz.vutbr.wsagent.util.TopologyDrawer;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import org.apache.struts2.ServletActionContext;

/**
 * The action class for presenting topology graphs.
 * @author Martin Gábor
 */
public class WSTopology extends ControlPanelAction{

    // image extension
    private final static String IMAGE_EXTENSION = ".jpg";
    
    // standard request
    public final static String DEF_REQUEST = "$(n)$(m,(1))";

    private String moteId;
    private long paramDate = 0;
    private List topologyRecords;
    private String imageSource;


    /**
     * Execute method.
     */
    @Override
    public String execute() {


        // get list of dates
        List messageList = null;
        try {
            messageList = DAOFactory.getDAOFactory().getMessageDAO().topologyReadAllBS(BASEST_ADDR);
        } catch (DatabaseErrorException ex) {
            setResponseMessage(getText("Db.Error.message"));
            return SUCCESS;
        }

        RSSIParser parser = new RSSIParser();
        // set list of dates to the session
        setTopologyRecords(parser.getCommonRssiDates(messageList));

        if (topologyRecords.isEmpty())
            return SUCCESS;

        /* GETTING THE TOPOLOGY GRAPH */
        // get concrete record with motes position
        RSSIRecord rssiRecord = (RSSIRecord) topologyRecords.get((int) paramDate);
        // get date
        String date= ""+rssiRecord.getDate();
        // source from settings
        String sourcePath = DAOFactory.getDAOFactory().getSettingDAO().read().getTopologyImgPath();

        /* GETTING THE SOURCE PATH*/
        // the absolute webapp path
        String input = ServletActionContext.getServletContext().getRealPath("");

        // the path of image;
        String imagePath = sourcePath+date+IMAGE_EXTENSION;
        setImageSource(imagePath);

        // if the file exists return with SUCCESS.
        if(new File(input+imagePath).exists())
            return SUCCESS;


        /*
         * Compute topology
         * ------------------
         */
        TopologyMath math = new TopologyMath();
        List objectList = new ArrayList();
        try {

            objectList = math.getTopologyPoints(rssiRecord.getRssiObjectList());

        } catch (InvalidInputStrengthsException ex) {
            setResponseMessage(getText("UnableLoadImage.message"));
            return SUCCESS;
        }

        // get border
        RectangleObject border = math.getWindowBorder(objectList);
        int width = border.getX2() - border.getX1();
        int height = border.getY2() - border.getY1();
        // this shouldnt happen, just in case return SUCCESS
        if(width <= 0 || height <= 0)
            return SUCCESS;

        /*--------------------*/
        // DRAW TOPOLOGY

        // get topology records for concrete Date (the value of paramDate)
        TopologyDrawer drawer = new TopologyDrawer(width, height, input);
        try {
            drawer.drawTopology(objectList, border);
            drawer.generateTopologyImage(imagePath);
        } catch (IOException ex) {
            setResponseMessage(getText("InvalidImagesSourcePath.message"));
        } catch (NullPointerException ex) {
            setResponseMessage(getText("InvalidImagesSourcePath.message"));
        }
        

        return SUCCESS;
    }

    /**
     * Get the value of moteId
     *
     * @return the value of moteId
     */
    public String getMoteId() {
        return moteId;
    }

    /**
     * Set the value of moteId
     *
     * @param moteId new value of moteId
     */
    public void setMoteId(String moteId) {
        this.moteId = moteId;
    }

    /**
     * Get the value of paramDate
     *
     * @return the value of paramDate
     */
    public long getParamDate() {
        return paramDate;
    }

    /**
     * Set the value of paramDate
     *
     * @param paramDate new value of paramDate
     */
    public void setParamDate(long paramDate) {
        this.paramDate = paramDate;
    }


    /**
     * Get the value of topologyRecords
     *
     * @return the value of topologyRecords
     */
    public List getTopologyRecords() {
        return topologyRecords;
    }

    /**
     * Set the value of topologyRecords
     *
     * @param topologyRecords new value of topologyRecords
     */
    public void setTopologyRecords(List topologyRecords) {
        this.topologyRecords = topologyRecords;
    }

    /**
     * Get the value of imageSource
     *
     * @return the value of imageSource
     */
    public String getImageSource() {
        return imageSource;
    }

    /**
     * Set the value of imageSource
     *
     * @param imageSource new value of imageSource
     */
    public void setImageSource(String imageSource) {
        this.imageSource = imageSource;
    }

    /**
     * Get the value of DEF_REQUEST
     *
     * @return the value of DEF_REQUEST
     */
    public String getDEF_REQUEST() {
        return DEF_REQUEST;
    }


}
