// -----------------------------------------------------------
// WSTracingAgents.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.controlpanel;

import cz.vutbr.wsagent.db.hibernate.DAOFactory;
import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import cz.vutbr.wsagent.util.LOGParser;
import cz.vutbr.wsagent.entity.Mote;
import cz.vutbr.wsagent.entity.Message;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * The action class for classification of colledted agents
 * @author Marek Houšť
 */
public class WSTracingAgents extends ControlPanelAction {

    // for identification whether it came as messages with information on the surrounding neighbors
    private static final String RSSI_PATTERN_FOR_TRACES = "\\(M,[0-9 ]+,[0-9 ]+,[0-9 ]+\\)";
    // list with collected logs
    private List logsRecords;
    private List tracingAgentList;
    private List motesAddress;

    private int sendOk;
    private boolean drawOk;
    private long numberSignalMessage;
    private long numberLogMessage;

    /**
     * Execute method.
     */
    @Override
    public String execute() {

        drawOk = false;

        if (sendOk == 1) {
            setResponseMessage(getText("Comm.Success.tracingagentsend"));
        } else if (sendOk == 2) {
            setResponseMessage(getText("Comm.TimeExpired.tracingagent"));
        }


        // get list of dates
        List messageList = null;
            
        try {
            messageList = DAOFactory.getDAOFactory().getMessageDAO().logReadAllBS(BASEST_ADDR);
        } catch (DatabaseErrorException ex) {
            setResponseMessage(getText("Db.Error.message"));
            return SUCCESS;
        }

        if(messageList.isEmpty()) {
            return SUCCESS;
        }

        numberSignalMessage = 0;
        // check if in messageList is RSSI data
        Pattern pattern = Pattern.compile(RSSI_PATTERN_FOR_TRACES);
        for (int i = 0; i < messageList.size(); i++) {

            Message message = (Message) messageList.get(i);

            if (message == null || message.getValue() == null) {
                continue;
            }
            // erase empty spaces
            String mess = message.getValue();
            message.setValue(mess.replaceAll(" ", ""));
            
            // check for pattern
            Matcher m = pattern.matcher(message.getValue());
            
            if (m.find()) {
                // tracingAgent can be rendered
                drawOk = true;
                numberSignalMessage = numberSignalMessage + 1;
            }
        }

        // class LOGParser is for parsing new message with logs
        LOGParser logParser = new LOGParser();
        // fill logsRecords
        setLogsRecords(logParser.getCommonLogDates(messageList));
        setNumberLogMessage(logsRecords.size());
        // fill motesAddress
        setMotesAddress(logParser.getMotesAddressFromLogs(logsRecords));
        // fill tracingAgentList
        setTracingAgentList(logParser.getTracingAgentList(logsRecords));

        if (logsRecords.isEmpty()) {
            return SUCCESS;
        }
        
        /*
         * Save online motes to database.
         */
        saveOnlineMotes(motesAddress);
        
        return SUCCESS;
    }

    /**
     * Set the value of logsRecords
     *
     * @param logsRecords new value of logsRecords
     */
    public void setLogsRecords(List logsRecords) {
        this.logsRecords = logsRecords;
    }

    /**
     * Set the value of motesAddress
     *
     * @param motesAddress new value of motesAddress
     */
    public void setMotesAddress(List motesAddress) {
        this.motesAddress = motesAddress;
    }

    /**
     * Get the value of tracingAgentList
     *
     * @return the value of tracingAgentList
     */
    public List getTracingAgentList() {
        return tracingAgentList;
    }

    /**
     * Set the value of tracingAgentList
     *
     * @param tracingAgentList new value of tracingAgentList
     */
    public void setTracingAgentList(List tracingAgentList) {
        this.tracingAgentList = tracingAgentList;
    }

    /**
     * Get the value of sendOk
     *
     * @return the value of sendOk
     */
    public int getSendOk() {
        return sendOk;
    }

    /**
     * Set the value of sendOk
     *
     * @param sendOk new value of sendOk
     */
    public void setSendOk(int sendOk) {
        this.sendOk = sendOk;
    }

    /**
     * Get the value of drawOk
     *
     * @return the value of drawOk
     */
    public boolean getDrawOk() {
        return drawOk;
    }

    /**
     * Set the value of drawOk
     *
     * @param drawOk new value of drawOk
     */
    public void setDrawOk(boolean drawOk) {
        this.drawOk = drawOk;
    }

    /**
     * Get the value of numberSignalMessage
     *
     * @return the value of numberSignalMessage
     */
    public long getNumberSignalMessage() {
        return numberSignalMessage;
    }

    /**
     * Set the value of numberSignalMessage
     *
     * @param numberSignalMessage new value of numberSignalMessage
     */
    public void setNumberSignalMessage(long numberSignalMessage) {
        this.numberSignalMessage = numberSignalMessage;
    }

    /**
     * Get the value of numberLogMessage
     *
     * @return the value of numberLogMessage
     */
    public long getNumberLogMessage() {
        return numberLogMessage;
    }

    /**
     * Set the value of numberLogMessage
     *
     * @param numberLogMessage new value of numberLogMessage
     */
    public void setNumberLogMessage(long numberLogMessage) {
        this.numberLogMessage = numberLogMessage;
    }

    /**
     * Method for storing the collected addresses of nodes.
     * @param motesAddress
     */
    private void saveOnlineMotes(List motesAddress) {

        for(int i = 0; i < motesAddress.size(); i++) {

            Mote mote = null;
            try {
                // for test if mote is already in the database
                mote = DAOFactory.getDAOFactory().getMoteDAO().read(motesAddress.get(i).toString());

            } catch (DatabaseErrorException ex) {
                setResponseMessage(getText("Db.Error.message"));
            }
            // test if mote is already in the database
            if (mote == null) {
                // if not
                mote = new Mote();
                mote.setAddress(motesAddress.get(i).toString());
                mote.setDescription("added by tracing agents");
                DAOFactory.getDAOFactory().getMoteDAO().save(mote);
            }
        }
    }

}
