// -----------------------------------------------------------
// CheckNewMessages.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.controlpanel.system;

import cz.vutbr.wsagent.controlpanel.*;
import cz.vutbr.wsagent.db.hibernate.DAOFactory;
import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import cz.vutbr.wsagent.entity.Message;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.List;

/**
 * The action class for checking new messages in the system. Makes database queries
 * to check if any message has come.
 * @author Martin Gábor
 */
public class CheckNewMessages extends ControlPanelAction {

    private boolean isNewMessage;
    private Message newMessage;
    private boolean isEndTracingAgents;
    private boolean isNewLogInDatabase;
    private static final String END_TRACINGAGENTS = "endlog";
    private static final String LOG_PATTERN = "\\(LOG,[0-9 ]+,[0-9 ]+,[0-9 ]+,[0-9 ]+,[0-9 ]+\\)";

    /**
     * Execute method.
     */
    @Override
    public String execute() {

        List messages = null;
        try {
            messages = DAOFactory.getDAOFactory().getMessageDAO().readNew();
            
        } catch (DatabaseErrorException ex) {
            return SUCCESS;
        }

        if (messages == null || messages.isEmpty()) {
            
            isNewMessage = false;
            isEndTracingAgents = false;
            isNewLogInDatabase = false;

        } else {

            isNewMessage = true;
            setNewMessage((Message) messages.get(0));

            Pattern endLogPattern = Pattern.compile(END_TRACINGAGENTS);
            Pattern logPattern = Pattern.compile(LOG_PATTERN);
            isEndTracingAgents = false;
            isNewLogInDatabase = false;

            for (int i = 0; i < messages.size(); i++) {

                Message message = (Message) messages.get(i);

                if (message == null || message.getValue() == null) {
                    continue;
                }

                Matcher endLogMatcher = endLogPattern.matcher(message.getValue());
                Matcher logMatcher = logPattern.matcher(message.getValue());
                if (endLogMatcher.find()) {

                    isEndTracingAgents = true;
                    isNewLogInDatabase = false;
                    break;
                } else if (logMatcher.find()) {

                    isNewLogInDatabase = true;
                }
            }
        }

        return SUCCESS;
    }

    /**
     * Get the value of isNewMessage
     *
     * @return the value of isNewMessage
     */
    public boolean isIsNewMessage() {
        return isNewMessage;
    }

    /**
     * Set the value of isNewMessage
     *
     * @param isNewMessage new value of isNewMessage
     */
    public void setIsNewMessage(boolean isNewMessage) {
        this.isNewMessage = isNewMessage;
    }

    /**
     * Get the value of isEndTracingAgents
     *
     * @return the value of isEndTracingAgents
     */
    public boolean isIsEndTracingAgents() {
        return isEndTracingAgents;
    }

    /**
     * Set the value of isEndTracingAgents
     *
     * @param isEndTracingAgents new value of isEndTracingAgents
     */
    public void setIsEndTracingAgents(boolean isEndTracingAgents) {
        this.isEndTracingAgents = isEndTracingAgents;
    }

    /**
     * Get the value of isNewLogInDatabase
     *
     * @return the value of isNewLogInDatabase
     */
    public boolean isIsNewLogInDatabase() {
        return isNewLogInDatabase;
    }

    /**
     * Set the value of isNewLogInDatabase
     *
     * @param isNewLogInDatabase new value of isNewLogInDatabase
     */
    public void setIsNewLogInDatabase(boolean isNewLogInDatabase) {
        this.isNewLogInDatabase = isNewLogInDatabase;
    }

    /**
     * Get the value of newMessage
     *
     * @return the value of newMessage
     */
    public Message getNewMessage() {
        return newMessage;
    }

    /**
     * Set the value of newMessage
     *
     * @param newMessage new value of newMessage
     */
    public void setNewMessage(Message newMessage) {
        this.newMessage = newMessage;
    }
}
