// -----------------------------------------------------------
// CheckOnlineMotes.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.controlpanel.system;

import cz.vutbr.wsagent.client.ClientSocket;
import cz.vutbr.wsagent.client.MessageConstructor;
import cz.vutbr.wsagent.client.TimeExpiredException;
import cz.vutbr.wsagent.controlpanel.ControlPanelAction;
import cz.vutbr.wsagent.db.hibernate.DAOFactory;
import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import cz.vutbr.wsagent.entity.Mote;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * The action class for sending empty messages to all motes of system to
 * check if it is active.
 * @author Martin Gábor
 */
public class CheckOnlineMotes extends ControlPanelAction{
    
    private List onlineMotes;

    /**
     * Execute method.
     */
    @Override
    public String execute() {

        // init list of online motes
        onlineMotes = new ArrayList<Mote>();

        // gets port
        int port = DAOFactory.getDAOFactory().getSettingDAO().read().getPort();

        // creates new message according to protocol
        MessageConstructor messageConstructor = new MessageConstructor();

        // list of all motes
        List moteList = null;
        // getting all Motes from DB
        try{
            moteList = DAOFactory.getDAOFactory().getMoteDAO().readAll();
        }catch(DatabaseErrorException e){
            return SUCCESS;
        }

        // creating new connector
        ClientSocket clientSocket = null;



        // looking for motes that are online
        // --------------------------------------------------------------
        for (Iterator mote = moteList.iterator(); mote.hasNext();) {
            Mote existingMote = (Mote) mote.next();

            //String messageToSend = messageConstructor.constructSimpleMessage(Integer.valueOf(existingMote.getAddress()), "");
            String messageToSend = messageConstructor.contructRefreshMessage(Integer.valueOf(existingMote.getAddress()));

            try {
                clientSocket = new ClientSocket(port);
                clientSocket.send(messageToSend);
                clientSocket.close();

                onlineMotes.add(existingMote);
            } catch (TimeExpiredException e){
                continue;
            } catch (Exception e){
                break;
            }
        }
        // ---------------------------------------------------------------

        return SUCCESS;

    }


    /**
     * Get the value of onlineMotes
     *
     * @return the value of onlineMotes
     */
    public List<Mote> getOnlineMotes() {
        return onlineMotes;
    }

    /**
     * Set the value of onlineMotes
     *
     * @param onlineMotes new value of onlineMotes
     */
    public void setOnlineMotes(List onlineMotes) {
        this.onlineMotes = onlineMotes;
    }

}
