// -----------------------------------------------------------
// DAOFactory.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.db.hibernate;

import cz.vutbr.wsagent.entity.IDAOFactory;
import cz.vutbr.wsagent.entity.IMessageDAO;
import cz.vutbr.wsagent.entity.IMoteDAO;

/**
 * The factory class for getting DAO objects.
 * @author Martin Gábor
 */
public class DAOFactory implements IDAOFactory {

    private static DAOFactory daoFactory = null;

    /**
     * Empty constructor for creating singleton instance.
     */
    private DAOFactory(){
    }

    /**
     * This is used for getting the instance of class. If there is no instance, new is created.
     */
    public static DAOFactory getDAOFactory() {
        if (daoFactory == null)
            daoFactory = new DAOFactory();
        return daoFactory;
    }

    /**
     * Returns agentDAO object.
     */
    public AgentDAO getAgentDAO() {
        AgentDAO agentDAO = new AgentDAO();
        return agentDAO;
    }

    /**
     * Returns getSettingDAO object.
     */
    public SettingDAO getSettingDAO() {
        SettingDAO settingDAO = new SettingDAO();
        return settingDAO;
    }

    /**
     * Returns getMoteDAO object.
     */
    public IMoteDAO getMoteDAO() {
        MoteDAO moteDAO = new MoteDAO();
        return moteDAO;
    }

    /**
     * Returns getMessageDAO object.
     */
    public IMessageDAO getMessageDAO() {
        MessageDAO messageDAO = new MessageDAO();
        return messageDAO;
    }
    
}
