// -----------------------------------------------------------
// SettingDAO.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.db.hibernate;

import cz.vutbr.wsagent.entity.ISettingDAO;
import cz.vutbr.wsagent.entity.Setting;
import org.hibernate.Query;
import org.hibernate.Session;

/**
 * This class implements ISettingDAO interface. It supports data access to Setting object of
 * hibernate framework.
 *
 * @author Martin Gábor
 */
public class SettingDAO implements ISettingDAO{

    // DEFAULT port
    private static final int DEF_PORT = 7777;
    // DEFAULT timeperiod
    private static final int DEF_TIME_PERIOD = 10000;
    // DEFAULT path
    private static final String DEF_TOPOLOGY_PATH = "/topologyImg/";

    Session session = null;

    /**
     * Constructor for creating hibernate session using <code>DbConnFactory</code>.
     */
    public SettingDAO() {
        this.session = DbConnFactory.getSession();
    }

    /**
     * This method is used for storing current settings to hibernate database.
     *
     * @param setting Setting.
     * @see Setting
     */
    public void save(Setting setting) {
        session.beginTransaction();
        session.delete(setting);
        session.saveOrUpdate(setting);
        session.flush();
    }

    /**
     * This method is used to get settings from hibernate database.
     *
     * @return Returns the current setting.
     * @see Setting
     */
    public Setting read(){
        Setting setting = null;
        try {
            session.beginTransaction();
            Query q = session.createQuery("from Setting");
            setting = (Setting) q.uniqueResult();
        } catch (Exception e) {
        }

        // if no settings in database
        if (setting == null){
            // new
            setting = new Setting();
            
            // set all defaults
            setting.setPort(DEF_PORT);
            setting.setTimePeriodActiveMotes(DEF_TIME_PERIOD);
            setting.setTimePeriodNewMessages(DEF_TIME_PERIOD);
            setting.setTopologyImgPath(DEF_TOPOLOGY_PATH);
        }

        return setting;
    }

}
