// -----------------------------------------------------------
// IAgentDAO.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.entity;

import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import java.util.List;

/**
 * Interface for storing agent object to database.
 *
 * @author Martin Gábor
 * @see Agent
 */
public interface IAgentDAO {


    /**
     * This method is used for storing new agent to database.
     *
     * @param agent Agent.
     * @see Agent
     */
    void save(Agent agent);

    /**
     * This method is used for storing new agent to database.
     *
     * @param agent Agent.
     * @see Agent
     */
    void update(Agent agent);


    /**
     * This method is used to delete agent from database.
     *
     * @param agent Agent.
     * @see Agent
     */
    void delete(Agent agent);


    /**
     * This method is used to get all agents from database.
     *
     * @return Returns list of all agents.
     * @see Agent
     */
    List<Agent> readAll() throws DatabaseErrorException;

    /**
     * This method is used to get concrete agent from database.
     *
     * @param agentId Id of agent.
     * @return Returns concrete agent.
     * @see Agent
     */
    Agent read(long agentId) throws DatabaseErrorException;

    /**
     * This method is used to get concrete agent from database.
     *
     * @param agentId Agent id of agent.
     * @param classId Agent class id of agent.
     * @return Returns concrete agent.
     * @see Agent
     */
    Agent read(long agentId, long classId) throws DatabaseErrorException;
}
