// -----------------------------------------------------------
// IMessageDAO.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.entity;

import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import java.util.List;

/**
 * Interface for storing message object to database.
 *
 * @author martin
 */
public interface IMessageDAO {

    /**
     * Saves the message bean to the database.
     * @param message Message to be saved.
     */
    void save(Message message);

    /**
     * Delete  message bean from database.
     * @param message Message to be deleted.
     */
    void delete(Message message);

    /**
     * This method is used to get all messages from database.
     *
     * @return Returns list of all messages.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Message
     */
    List<Message> readAll() throws DatabaseErrorException;

    /**
     * This method is used to get all messages from database for topology purposes.
     *
     * @param BSAddress The address of basestation.
     * @return Returns list of all messages.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Message
     */
    List<Message> topologyReadAllBS(String BSAddress) throws DatabaseErrorException;

    /**
     * This method is used to get all messages for concrete mote from database.
     *
     * @param mote the address of mote.
     * @return Returns list of all messages.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Message
     */
    List<Message> readAllForMote(Mote mote) throws DatabaseErrorException;

    /**
     * This method is used to get all new messages from database.
     *
     * @return Returns list of all messages.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Message
     */
    List<Message> readNew() throws DatabaseErrorException;

    /**
     * This method is used to get particular message from database.
     *
     * @param id Id of required message.
     * @return Returns the message of given id.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Message
     */
    public Message read(long id) throws DatabaseErrorException;

    /**
     * This method is used to update concrete new messages.
     *
     * @param message The message.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Message
     */
    public void updateNew(Message message) throws DatabaseErrorException;

    /**
     * This method is used to update new messages for particular mote.
     *
     * @param mote The address of mote.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Message
     */
    void updateNew(Mote mote) throws DatabaseErrorException;

    /**
     * This method is used to update all new messages in database.
     *
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Message
     */
    void updateNewAll() throws DatabaseErrorException;

    /**
     * This method is used to get all messages from database for tracing agents purposes.
     *
     * @param BSAddress The address of basestation.
     * @return Returns list of all messages.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Message
     */
    List<Message> logReadAllBS(String BSAddress) throws DatabaseErrorException;
    
}
