// -----------------------------------------------------------
// AgentPoint.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.util;
import java.util.List;

/**
 * Class for one agent movement (counter).
 * @author Marek Houšť
 */
public class AgentPoint extends GeometryObject {

    private int x;
    private int y;
    private long from;
    private long to;
    private List<Long> listCounter;

    /**
     * The constructor of agent point object.
     * We need line coordinate because of the location. Central point is not enough.
     * Agent point is separated from the trace line because at the agent point depends on the number.
     * Trace line is not moved with the number. Agent point is moved with the number.
     * @param x1 X-coordinate of start point line witch agent point is.
     * @param y1 Y-coordinate of start point line witch agent point is.
     * @param x2 X-coordinate of end point line witch agent point is.
     * @param y2 X-coordinate of end point line witch agent point is.
     * @param from indicates line between the agent point is
     * @param to indicates line between the agent point is
     * @param listCounter list of counter text by line.
     */
    public AgentPoint(int temp_x1, int temp_y1, int temp_x2, int temp_y2, long from, long to, List<Long> listCounter, String name) {

        int x1 = 0; int y1 = 0; int x2 = 0; int y2 = 0; int x = 0; int y = 0;

        // coordinate correction
        if (name.equals(AGENTPOINTLARGERSMALER)) {

            if (Math.abs(temp_x1 - temp_x2) > 30) {
                if (Math.abs(temp_y1 - temp_y2) > 30) {
                    if((temp_x1 < temp_x2) && (temp_y1 < temp_y2)) {
                        x1 = temp_x1 + 15;
                        y1 = temp_y1 + 5;
                        x2 = temp_x2 - 5;
                        y2 = temp_y2 - 15;
                        // middle point of trace line is agent point
                        x = x1 + ((x2 - x1) / 2);
                        y = y1 + ((y2 - y1) / 2);
                        x = x + 5;
                    } else if((temp_x1 < temp_x2) && (temp_y1 >= temp_y2)) {
                        x1 = temp_x1 + 5;
                        y1 = temp_y1 - 15;
                        x2 = temp_x2 - 15;
                        y2 = temp_y2 + 5;
                        // middle point of trace line is agent point
                        x = x1 + ((x2 - x1) / 2);
                        y = y1 + ((y2 - y1) / 2);
                        x = x - 30;
                    } else if((temp_x1 >= temp_x2) && (temp_y1 < temp_y2)) {
                        x1 = temp_x1 - 15;
                        y1 = temp_y1 + 5;
                        x2 = temp_x2 + 5;
                        y2 = temp_y2 - 15;
                        // middle point of trace line is agent point
                        x = x1 + ((x2 - x1) / 2);
                        y = y1 + ((y2 - y1) / 2);
                        x = x - 30;
                    } else if((temp_x1 >= temp_x2) && (temp_y1 >= temp_y2)) {
                        x1 = temp_x1 - 5;
                        y1 = temp_y1 - 15;
                        x2 = temp_x2 + 15;
                        y2 = temp_y2 + 5;
                        // middle point of trace line is agent point
                        x = x1 + ((x2 - x1) / 2);
                        y = y1 + ((y2 - y1) / 2);
                        x = x + 5;
                    }
                } else {
                    x1 = temp_x1;
                    y1 = temp_y1;
                    x2 = temp_x2;
                    y2 = temp_y2;
                    // middle point of trace line is agent point
                    x = x1 + ((x2 - x1) / 2);
                    y = y1 + ((y2 - y1) / 2);
                    y = y - 13;
                }
            } else {
                if (temp_y1 < temp_y2) {
                    x1 = temp_x1 + 8;
                    y1 = temp_y1 + 15;
                    x2 = temp_x2 + 8;
                    y2 = temp_y2 - 15;
                    // middle point of trace line is agent point
                    x = x1 + ((x2 - x1) / 2);
                    y = y1 + ((y2 - y1) / 2);
                    x = x + 5;
                } else if (temp_y1 >= temp_y2) {
                    x1 = temp_x1 + 8;
                    y1 = temp_y1 - 15;
                    x2 = temp_x2 + 8;
                    y2 = temp_y2 + 15;
                    // middle point of trace line is agent point
                    x = x1 + ((x2 - x1) / 2);
                    y = y1 + ((y2 - y1) / 2);
                    x = x + 5;
                }
            }

        } else if (name.equals(AGENTPOINTSMALERLARGER)) {

            if (Math.abs(temp_x1 - temp_x2) > 30) {
                if (Math.abs(temp_y1 - temp_y2) > 30) {
                    if((temp_x1 < temp_x2) && (temp_y1 < temp_y2)) {
                        x1 = temp_x1 + 5;
                        y1 = temp_y1 + 15;
                        x2 = temp_x2 - 15;
                        y2 = temp_y2 - 5;
                        // middle point of trace line is agent point
                        x = x1 + ((x2 - x1) / 2);
                        y = y1 + ((y2 - y1)/2);
                        x = x - 30;
                        y = y + (13 * listCounter.size());
                    } else if((temp_x1 < temp_x2) && (temp_y1 >= temp_y2)) {
                        x1 = temp_x1 + 15;
                        y1 = temp_y1 - 5;
                        x2 = temp_x2 - 5;
                        y2 = temp_y2 + 15;
                        // middle point of trace line is agent point
                        x = x1 + ((x2 - x1) / 2);
                        y = y1 + ((y2 - y1)/2);
                        y = y + (13 * listCounter.size());
                    } else if((temp_x1 >= temp_x2) && (temp_y1 < temp_y2)) {
                        x1 = temp_x1 - 5;
                        y1 = temp_y1 + 15;
                        x2 = temp_x2 + 15;
                        y2 = temp_y2 - 5;
                        // middle point of trace line is agent point
                        x = x1 + ((x2 - x1) / 2);
                        y = y1 + ((y2 - y1)/2);
                        y = y + (13 * listCounter.size());
                    } else if((temp_x1 >= temp_x2) && (temp_y1 >= temp_y2)) {
                        x1 = temp_x1 - 15;
                        y1 = temp_y1 - 5;
                        x2 = temp_x2 + 5;
                        y2 = temp_y2 + 15;
                        // middle point of trace line is agent point
                        x = x1 + ((x2 - x1) / 2);
                        y = y1 + ((y2 - y1)/2);
                        x = x - 30;
                        y = y + (13 * listCounter.size());
                    }
                } else {
                    x1 = temp_x1;
                    y1 = temp_y1;
                    x2 = temp_x2;
                    y2 = temp_y2;
                    // middle point of trace line is agent point
                    x = x1 + ((x2 - x1) / 2);
                    y = y1 + ((y2 - y1)/2);
                    y = y + 10 + (13 * listCounter.size());
                }
            } else {
                if (temp_y1 < temp_y2) {
                    x1 = temp_x1 - 8;
                    y1 = temp_y1 + 15;
                    x2 = temp_x2 - 8;
                    y2 = temp_y2 - 15;
                    // middle point of trace line is agent point
                    x = x1 + ((x2 - x1) / 2);
                    y = y1 + ((y2 - y1) / 2);
                    x = x - 30;
                    y = y + (13 * listCounter.size());
                } else if (temp_y1 >= temp_y2) {
                    x1 = temp_x1 - 8;
                    y1 = temp_y1 - 15;
                    x2 = temp_x2 - 8;
                    y2 = temp_y2 + 15;
                    // middle point of trace line is agent point
                    x = x1 + ((x2 - x1) / 2);
                    y = y1 + ((y2 - y1) / 2);
                    x = x - 30;
                    y = y + (13 * listCounter.size());
                }
            }

        }

        this.x = x;
        this.y = y;
        this.from = from;
        this.to = to;
        this.listCounter = listCounter;
        this.name = AGENTPOINT;

    }

    /**
     * Get the value of x
     *
     * @return the value of x
     */
    public int getX() {
        return x;
    }

    /**
     * Set the value of x
     *
     * @param x new value of x
     */
    public void setX(int x) {
        this.x = x;
    }

    /**
     * Get the value of y
     *
     * @return the value of y
     */
    public int getY() {
        return y;
    }

    /**
     * Set the value of y
     *
     * @param y new value of y
     */
    public void setY(int y) {
        this.y = y;
    }

    /**
     * Get the value of from
     *
     * @return the value of from
     */
    public long getFrom() {
        return from;
    }

    /**
     * Set the value of from
     *
     * @param from new value of from
     */
    public void setFrom(long from) {
        this.from = from;
    }

    /**
     * Get the value of to
     *
     * @return the value of to
     */
    public long getTo() {
        return to;
    }

    /**
     * Set the value of to
     *
     * @param to new value of to
     */
    public void setTo(long to) {
        this.to = to;
    }

    /**
     * Get the list of counters
     *
     * @return the list of counters
     */
    public List<Long> getListCounter() {
        return listCounter;
    }

    /**
     * Set the list of listCounter
     *
     * @param counter new list of counters
     */
    public void setListCounters(List<Long> listCounter) {
        this.listCounter = listCounter;
    }

}
