// -----------------------------------------------------------
// CircleObject.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.util;

import java.util.ArrayList;
import java.util.List;

/**
 * The class describing simple circle.
 * @author Martin Gábor.
 */
public class CircleObject extends GeometryObject{

    private int x;
    private int y;
    private int r;

  /**
   * The constructor for creating new instance of circle object with given parameters.
   * @param  x The X-coordinate.
   * @param  y The Y-coordinate.
   * @param r The radius.
   */
    public CircleObject(int x, int y, int r){

        this.x = x;
        this.y = y;
        this.r = r;
        this.name="circle";

    }

        /**
     * Get the value of x
     *
     * @return the value of x
     */
    public int getX() {
        return x;
    }

    /**
     * Set the value of x
     *
     * @param x new value of x
     */
    public void setX(int x) {
        this.x = x;
    }

        /**
     * Get the value of y
     *
     * @return the value of y
     */
    public int getY() {
        return y;
    }

    /**
     * Set the value of y
     *
     * @param y new value of y
     */
    public void setY(int y) {
        this.y = y;
    }

    /**
     * Get the value of r
     *
     * @return the value of r
     */
    public int getR() {
        return r;
    }

    /**
     * Set the value of r
     *
     * @param r new value of r
     */
    public void setR(int r) {
        this.r = r;
    }


    /**
     * Determines intersections of this circle and a circle given as a parameter.
     * @param circle2 The second circle.
     * @return Returns instances of PointObject.
     */
     public List<PointObject> getIntersection(CircleObject circle2) {

        double dx, dy;
        double d, d2;
        double a, h;
        double x2, y2;
        double firstX, firstY;
        double secondX, secondY;

        double aX = this.getX();
        double aY = this.getY();
        double aR = this.getR();
        double bX = circle2.getX();
        double bY = circle2.getY();
        double bR = circle2.getR();

        List resultList = new ArrayList<PointObject>();

        dx = aX - bX;
	dy = aY - bY;

        d2 = dx*dx + dy*dy;
	d = Math.sqrt( d2 );

	if ( d > (aR + bR))
            return resultList; // no solution
        if(d < Math.abs(aR-bR))
            return resultList;


	a = ((aR*aR) - (bR*bR) + d2) / (2*d);
	h = Math.sqrt((aR*aR) - (a * a));
	x2 = aX + a*(bX - aX)/d;
	y2 = aY + a*(bY - aY)/d;


        /* Determine the absolute intersection points. */
	firstX = x2 + h*(bY - aY)/d;
	firstY = y2 - h*(bX - aX)/d;
	secondX = x2 - h*(bY - aY)/d;
	secondY = y2 + h*(bX - aX)/d;


        // add to the result points
        PointObject resultPoint1 = new PointObject((int) firstX, (int) firstY, "");
        PointObject resultPoint2 = new PointObject((int) secondX, (int) secondY, "");


        resultList.add(resultPoint1);
        resultList.add(resultPoint2);

        // return intersections
        return resultList;
    }


    /**
     * Increase a radius of circle object.
     * @param increment value of increment.
     */
    public void increaseRadius(int increment){
        r = r + increment;
    }

    /**
     * Decrease a radius of circle object.
     * @param decrement value of decrement.
     */
    public void decreaseRadius(int decrement){
        r = r - decrement;
    }
}
