// -----------------------------------------------------------
// PointObject.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.util;

/**
 * The class describing simple point.
 * @author Martin Gábor.
 */
public class PointObject extends GeometryObject {

    private static final double DIST_APPROX = 40;
    private int x;
    private int y;
    private String id = null;

  /**
   * The constructor for creating new instance of point object with given parameters.
   * @param  x The X-coordinate.
   * @param  y The Y-coordinate.
   * @param id Id of point, address.
   */
    public PointObject(int x, int y, String id) {

        this.x = x;
        this.y = y;
        this.id = id;
        this.name = "point";

    }

    /**
     * Get the value of x
     *
     * @return the value of x
     */
    public int getX() {
        return x;
    }

    /**
     * Set the value of x
     *
     * @param x new value of x
     */
    public void setX(int x) {
        this.x = x;
    }

    /**
     * Get the value of y
     *
     * @return the value of y
     */
    public int getY() {
        return y;
    }

    /**
     * Set the value of y
     *
     * @param y new value of y
     */
    public void setY(int y) {
        this.y = y;
    }

    /**
     * Get the value of id
     *
     * @return the value of id
     */
    public String getId() {
        return id;
    }

    /**
     * Set the value of id
     *
     * @param id new value of id
     */
    public void setId(String id) {
        this.id = id;
    }

    /**
     * Method for determining distance.
     * @param point2 The second point.
     * @return Returns value of distance.
     */
    public int determineDistance(PointObject point2) {

        double dx, dy;

        dx = this.getX() - point2.getX();
        dy = this.getY() - point2.getY();

        return (int) Math.sqrt(dx * dx + dy * dy);
    }

    /**
     * Returns true if it is the same point.
     * @param point2 The second point.
     * @return Returns true if it is the same point.
     */
    public boolean isTheSamePoint(PointObject point2) {


        double distance = determineDistance(point2);

        if (distance < DIST_APPROX) {
            return true;
        } else {
            return false;
        }

    }
}
