// -----------------------------------------------------------
// TraceMath.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

/**
 * Class to calculate the plotted objects.
 * @author Marek Houšť
 */
public class TraceMath {

    /**
     * Method for verifying the correct coordinates (coordinates have been calculated by class TopologyMath)
     * @param width
     * @param height
     * @throws InvalidInputStrengthsException
     */
    public void checkWidthHeight(int width, int height) throws InvalidInputStrengthsException {

        // this shouldnt happen
        if(width <= 0 || height <= 0) {
            List<Long> a = null;
            try {
                a.add(Long.MAX_VALUE);
            } catch (NullPointerException e) {
                throw new InvalidInputStrengthsException();
            }
        }

    }

    /**
     * Method for conversion points to plotted objects.
     * @param tracingAgent
     * @param moteList
     * @return resultPoints
     * @throws InvalidInputStrengthsException
     */
    public List<GeometryObject> getTracePoints(TracingAgent tracingAgent, List<GeometryObject> moteList) throws InvalidInputStrengthsException {

        // converting visited and not visited points
        List resultPoints = CopyAndSetTracePoint(moteList, tracingAgent.getMotesAddressWhereWas());

        // movements of the selected agent
        List traceList = tracingAgent.getTraceList();
        Trace trace = null;
        List<AgentPoint> listAgentPoints = new ArrayList();

        // movements of the selected agent are browsed
        for (Iterator traceListI = traceList.iterator(); traceListI.hasNext();) {

            trace = (Trace) traceListI.next();
            boolean findMotesForTraceLine = false;
            TracePoint fromPoint = null;
            TracePoint toPoint = null;

            // starting point jump
            fromPoint = findPoint(resultPoints, trace.getFrom());
            if(fromPoint != null) {
                findMotesForTraceLine = true;
            }

            if(findMotesForTraceLine) {
                // end point jump
                toPoint = findPoint(resultPoints, trace.getTo());
            }
            if(toPoint != null) {
                findMotesForTraceLine = true;
            }

            // we will paint trace line
            if (findMotesForTraceLine) {
                TraceLine traceLine = null;
                
                // if agent goes from mote with smaller address to larger
                // we are painting bottom trace line
                if (Long.parseLong(fromPoint.getId()) < Long.parseLong(toPoint.getId())) {

                    traceLine = new TraceLine(fromPoint.getX(), fromPoint.getY(), toPoint.getX(), toPoint.getY(), GeometryObject.TRACELINESMALERLARGER);

                    // we need it for update agent point
                    AgentPoint temp_agentPoint = null;
                    int i = 0;
                    boolean findSameAgentPoint = false;
                    
                    // we look at if for actual trace line exists agent point
                    for (i = 0; i < listAgentPoints.size(); i++) {
                        temp_agentPoint = listAgentPoints.get(i);
                        if ((temp_agentPoint.getFrom() == trace.getFrom()) && (temp_agentPoint.getTo() == trace.getTo())) {
                            findSameAgentPoint = true;
                            break;
                        } else {
                            findSameAgentPoint = false;
                        }
                    }

                    if (findSameAgentPoint) {
                        // update old agent point
                        List<Long> temp_listCounter = temp_agentPoint.getListCounter();
                        temp_listCounter.add(trace.getCounter());

                        AgentPoint agentPoint = new AgentPoint(fromPoint.getX(), fromPoint.getY(), toPoint.getX(), toPoint.getY(), Long.parseLong(fromPoint.getId()), Long.parseLong(toPoint.getId()), temp_listCounter, GeometryObject.AGENTPOINTSMALERLARGER);
                        listAgentPoints.set(i, agentPoint);

                    } else {
                        // make new agent point
                        List<Long> listCounter = new ArrayList();
                        listCounter.add(trace.getCounter());

                        AgentPoint agentPoint = new AgentPoint(fromPoint.getX(), fromPoint.getY(), toPoint.getX(), toPoint.getY(), Long.parseLong(fromPoint.getId()), Long.parseLong(toPoint.getId()), listCounter, GeometryObject.AGENTPOINTSMALERLARGER);
                        listAgentPoints.add(agentPoint);
                    }

                } else {
                // if agent goes from mote with larger address to smaler
                // we are painting top trace line
                    
                    
                    traceLine = new TraceLine(fromPoint.getX(), fromPoint.getY(), toPoint.getX(), toPoint.getY(), GeometryObject.TRACELINELARGERSMALER);
                    // we need it for update agent point
                    AgentPoint temp_agentPoint = null;
                    int i = 0;
                    boolean findSameAgentPoint = false;
                    
                    // we look at if for actual trace line exists agent point
                    for (i = 0; i < listAgentPoints.size(); i++) {
                        temp_agentPoint = listAgentPoints.get(i);
                        if ((temp_agentPoint.getFrom() == trace.getFrom()) && (temp_agentPoint.getTo() == trace.getTo())) {
                            findSameAgentPoint = true;
                            break;
                        } else {
                            findSameAgentPoint = false;
                        }
                    }

                    if (findSameAgentPoint) {
                        // update old agent point
                        List<Long> temp_listCounter = temp_agentPoint.getListCounter();
                        temp_listCounter.add(trace.getCounter());

                        AgentPoint agentPoint = new AgentPoint(fromPoint.getX(), fromPoint.getY(), toPoint.getX(), toPoint.getY(), Long.parseLong(fromPoint.getId()), Long.parseLong(toPoint.getId()), temp_listCounter, GeometryObject.AGENTPOINTLARGERSMALER);
                        listAgentPoints.set(i, agentPoint);

                    } else {
                        // make new agent point
                        List<Long> listCounter = new ArrayList();
                        listCounter.add(trace.getCounter());

                        AgentPoint agentPoint = new AgentPoint(fromPoint.getX(), fromPoint.getY(), toPoint.getX(), toPoint.getY(), Long.parseLong(fromPoint.getId()), Long.parseLong(toPoint.getId()), listCounter, GeometryObject.AGENTPOINTLARGERSMALER);
                        listAgentPoints.add(agentPoint);
                    }

                }
                resultPoints.add(traceLine);
            }
            
        }

        for (int i = 0; i < listAgentPoints.size(); i++) {
            resultPoints.add(listAgentPoints.get(i));
        }

        return resultPoints;
    }

    /**
     * Method for finding a point of jump.
     * @param resultPoints
     * @param fromTo
     * @return point
     */
    private TracePoint findPoint(List<GeometryObject> resultPoints, long fromTo) {

        TracePoint point = null;
        TracePoint temp_point = null;
        
        for (int i = 0; i < resultPoints.size(); i++) {
                GeometryObject geoObject = (GeometryObject) resultPoints.get(i);
                if(geoObject.getName().compareTo(GeometryObject.TRACEPOINT) == 0) {
                    temp_point = (TracePoint) geoObject;

                    if (fromTo == Long.parseLong(temp_point.getId())) {
                        point = temp_point;
                        break;
                    }
                }
            }
        return point;
    }

    /**
     * Method for converting visited and not visited points.
     * @param moteList
     * @param listWhereAgentWas
     * @return resultPoint
     */
    private List<GeometryObject> CopyAndSetTracePoint(List<GeometryObject> moteList, List<Long> listWhereAgentWas) {

        List<GeometryObject> resultPoint = new ArrayList<GeometryObject>();
        PointObject pointObject = null;

        for (int i = 0; i < moteList.size(); i++) {
            GeometryObject geoObject = (GeometryObject) moteList.get(i);
            if(geoObject.getName().compareTo(GeometryObject.POINTNAME) == 0) {
                pointObject = (PointObject) geoObject;
                if(listWhereAgentWas.contains(Long.valueOf(pointObject.getId()))) {
                    TracePoint tracePoint = new TracePoint(pointObject.getX(), pointObject.getY(), pointObject.getId());
                    resultPoint.add(tracePoint);
                } else {
                    resultPoint.add(pointObject);
                }
            }
        }
        return resultPoint;
    }

}
